''
'' Major functionality
''
'' 1. Application can pass into Init a configuration table. The configuration table is 32 word entries. 1 per
'' pin, each entry contains a device type. Either from the standard list below, or from a app defined list,
'' see appconfig.spin for examples
'' 2. Applications should call config.GetPin(DeviceType) to get the pin for the device. GetPin will return -1 if
'' the device is not supported.
'' 3. Init checks for an in RAM table,the RAM table is at the top of RAM and contains a pin number for a small number
'' of standard devices. See XXX_OFFSET for the list of standard devices. An OS should place this table in RAM
'' when it loaded an application
'' 4. If a RAM table is not found, it checks if there is no device or standard devices on pins 12 and 13. If so it
'' runs a hardware detection and configures for keyboard/display based on that detection
''
CON
'standard pin definitions
'enumeration of standard devices
' Just add new devices to the end of the list
#0, NOT_USED
KEYBOARD1_DATA                                                                  'Primary keyboard
KEYBOARD1_CLK                                                                   'Primary keyboard
MOUSE1_DATA                                                                     'Primary mouse
MOUSE1_CLK                                                                      'Primary mouse
TV_DAC1                                                                         'Primary TV
VGA1                                                                            'Primary VGA
AUDIO1,AUDIO2                                                                   'Primary/Secondary Audio
FSRW1_DO,FSRW1_CLK,FSRW1_DI,FSRW1_CS                                            'Primary SD media connections
DS1302_INCLK,DS1302_INIO,DS1302_CS                                              'DS1302 clock
DEBUG_TX,DEBUG_RX                                                               'serial debug serial port
GPS_TX,GPS_RX                                                                   'serial gps serial port
CMUCAM_TX,CMUCAM_RX                                                             'cmucam serial port
XBEE_TX,XBEE_RX                                                                 'XBBE serial port
HYDRANET_TX,HYDRANET_RX                                                         'HYDRA Net
IRDETECT,IREMIT                                                                 'IR Emit/detect
PING_DATA                                                                       'Ping Sensor
JOY_CLK,JOY_SHLDN,JOY_DATAOUT0,JOY_DATAOUT1                                     'Nes controller configuration
PG_RXPIN,PG_TXPIN                                                               'PropGFX Lite configuration
DEBUG_LED1,DEBUG_LED2,DEBUG_LED3                                                'some leds for debugging
ENC1_SCK,ENC1_SI,ENC1_SO,ENC1_INT                                               'ENC28J60 Ethernet
HM55BEna,HM55BClk,HM55BDI,HM55BDO                                               'HM55B compass
I2C_SCL1, I2C_SDA1                                                              'primary I2C
I2C_SCL2, I2C_SDA2                                                              'secondary I2C
SERVO1, SERVO2, SERVO3,SERVO4                                                   'servos
SERVO5, SERVO6, SERVO7,SERVO8                                                   'servos
SERVO9, SERVO10, SERVO11,SERVO12                                                'servos
SERVO13, SERVO14, SERVO15,SERVO16                                               'servos
HB25_1,HB25_2,HB25_3,HB25_4                                                     'motor controllers
QE_1,QE_2,QE_3,QE_4                                                             'Quad encoder
QE_5,QE_6,QE_7,QE_8                                                             'Quad encoder
QE_9,QE_10,QE_11,QE_12                                                          'Quad encoder
QE_13,QE_14,QE_15,QE_16                                                         'Quad encoder
VEXDEMUX                                                                        'VEX RC receiver
HX512_C0,HX512_C1,HX512_CLK,HX512_RST,HX512_DATA                                'Hydra 512k expansion sram

'Magic number - change if you change the structure of the table
MAGICNUMBER      = $AAAAAAAA
'Looks are a table in RAM
'The address that is checked is STANDARDADDRESS
'The table is structured as 11 entries, 17bytes in length - see XXX_OFFSET
'the table contains a magic number, clock freq and clock mode
'Pins for a set of standard devices and NTSC/PAL configuration
'If the first entry contains the MAGICNUMBER then the table exists
'
'4 - magic no, 4 - clkfreq, 1 clkreg, 7 pins, 1 ntsc/pin
STANDARDADDRESS = $8000-4-4-1-7-1                                               'top of 32k eeprom
'offsets into table
MAGIC_OFFSET      = 0
CLKFREQ_OFFSET    = 4
CLKMODE_OFFSET    = 8
KEYBOARD_OFFSET   = 9
TV_DAC_OFFSET     = 10
VGA_OFFSET        = 11
FSRW1_DO_OFFSET   = 12
FSRW1_CLK_OFFSET  = 13
FSRW1_DI_OFFSET   = 14
FSRW1_CS_OFFSET   = 15
NTSC_PAL_OFFSET   = 16                                          

'NTSC/PAL values
NTSC             = 0
PAL              = 1

PUB Init(addr) | i,offset,index
''Takes a config table, then checks if a RAM table exists and updates the config table from it
''if no RAM table, and pins 12 and 13 are either not used or used by a standard set of device e.g. keyboard
''runs a hardware detect to detect demo board/hydra/hybrid
  PINS := addr
  if LONG[STANDARDADDRESS] == MAGICNUMBER               'see if we find a config table at ram location
    clkset(BYTE[STANDARDADDRESS+CLKMODE_OFFSET], LONG[STANDARDADDRESS+CLKFREQ_OFFSET])
    repeat offset from KEYBOARD_OFFSET to FSRW1_CS
      if (i := BYTE[STANDARDADDRESS+offset]) <> -1      'keyboard and other standard devices configured by OS?
        if CheckSpecial(i)
          'remove the device from the current table
          repeat index from 0 to 31
            if word[PINS][index] == word[@TYPE][offset-KEYBOARD_OFFSET]
              word[PINS][index] := NOT_USED
          'Add device - NOTE this only sets the first pin for a device 
          word[PINS][i] := word[@TYPE][offset-KEYBOARD_OFFSET]
    byte[PINS][64] := BYTE[STANDARDADDRESS+NTSC_PAL_OFFSET]
  elseif CheckSpecial(12) AND CheckSpecial(13)          'no table try hardware detect if pins 12 and 13 are used by standard devices
    HWDetect
    
PRI CheckSpecial(i) | ok, check
  ok := false
  repeat check from 0 to 8
    if SPECIAL[check] == word[PINS][i]
      ok := true
      quit
  return ok

' This routine is a modifed version of
' //////////////////////////////////////////////////////
' AUTHOR: Coley
' LAST MODIFIED: 23.06.08
' VERSION 1
' COMMENTS: Routine for checking Hardware configurations
'           of various Propeller based boards.             
' //////////////////////////////////////////////////////
'
'  Power on harware configurations for target boards
'
'                      Pin Number
'
'            00000000001111111111222222222233
'            01234567890123456789012345678901
'
'                                    vdac
' Hybrid     000000001111111100000000OOOO1111
'
'                        vdac
' Demoboard  000000000000OOOO0000000011110111
'
'                                    vdac
' Hydra      010001000101010111110000OOOO??11 
'
' For this application we will check the status
' of P12..P13
'
' %00 - Demo/Protoboard
' %01 - Hydra
' %11 - Hybrid
'
' //////////////////////////////////////////////////////
PRI  HWDetect | HWPins,PinTable,index,pcount
  PinTable := 0
  clkset(%01101000,  12_000_000)                        ' Set internal oscillator to RCFast and set PLL to start
  waitcnt(cnt+120_000)                                  ' wait approx 10ms at 12mhz for PLL to 'warm up'
  
  HWPins := INA[12..13]                                 ' Check state of Pins 12-13

  CASE HWPins
    %00 : clkset( %01101111, 80_000_000 )               ' Demo Board   80MHz (5MHz PLLx16)
          PinTable := @DEMO
    %01 : clkset( %01110110, 80_000_000 )               ' Hydra        80MHz (10MHz PLLx8)
          PinTable := @HYDRA
    %11 : clkset( %01101111, 96_000_000 )               ' Hybrid       96MHz (6MHz PLLx16)
          PinTable := @HYBRID
  if PinTable <> 0
    repeat index from 0 to 7
      repeat pcount from byte[PinTable][index<<1] to byte[PinTable][index<<1]+byte[PinTable][index<<1+1]-1
        word[PINS][pcount] := word[@TYPE][index<<1]
   
PUB GetPin(PinType) | index, addr
  repeat index from 0 to 31
    if word[PINS][index] == PinType
      return index
  return -1

PUB GetType(Pin)
'return whats on a pin
  return word[PINS][Pin]
  
PUB GetNTSCPAL
  return byte[PINS][64]
  
DAT
PINS          long 0
'do not change the order of SPECIAL and TYPE and their order with each other, Init routine depends on it
SPECIAL       word NOT_USED
TYPE          word KEYBOARD1_DATA,KEYBOARD1_CLK,TV_DAC1,VGA1,FSRW1_DO,FSRW1_CLK,FSRW1_DI,FSRW1_CS 'pins that are ok to overwrite
'pin and count info for various platforms in type order
DEMO          byte 26,1,27,1,12,4,16,8,0,0,0,0,0,0,0,0
HYDRA         byte 12,2,14,2,24,4,16,8,0,0,0,0,0,0,0,0
HYBRID        byte 12,2,14,2,24,4,0,0,8,1,9,1,10,1,11,1